#include "StdAfx.h"
#include "NWindowAdapter.h"

#include "ScriptletConstant.h"

REGISTER_COMPONENT_CREATOR(SCRIPTLET_WIN32_NODE, CNWindowAdapter);
UINT CNWindowAdapter::m_sCtrlId = 1000;

const static TCHAR NWINDOWADAPTER_PROP[] = _T("NWINDOWADAPTER_PROP");

IMPLEMENT_RTTI(CNWindowAdapter, CNControl) 

CNWindowAdapter::CNWindowAdapter()
{
	m_hWndAdaptee = NULL;
}

CNWindowAdapter::~CNWindowAdapter()
{
	DetachWindowAdaptee();
}

void CNWindowAdapter::AttachWindowAdaptee(HWND hWndAdaptee)
{
	ATLASSERT(::IsWindow(hWndAdaptee) == TRUE);
	if(::IsWindow(hWndAdaptee) == FALSE) return;

	m_hWndAdaptee = hWndAdaptee;

	::SetWindowLong(m_hWndAdaptee, GWL_ID, (LONG)m_sCtrlId);
	m_sCtrlId++;
}

void CNWindowAdapter::DetachWindowAdaptee()
{
	m_hWndAdaptee = NULL;
}
BOOL CNWindowAdapter::MoveWindow(LPRECT lpRect, BOOL bRepaint/*=FALSE*/)
{
	BOOL bResult = __super::MoveWindow(lpRect, bRepaint);
	if(::IsWindow(m_hWndAdaptee) == TRUE)
	{
		CPoint ptParentAbsolutePos;
		GetParentAbsolutePosition(&ptParentAbsolutePos);

		CRect rcBound;
		this->GetBounds(&rcBound);
		rcBound.OffsetRect(ptParentAbsolutePos);
		::MoveWindow(m_hWndAdaptee, rcBound.left, rcBound.top, rcBound.Width(), rcBound.Height(), bRepaint);		
	}
	return bResult;
}
BOOL CNWindowAdapter::SetActualBound(int nX, int nY, int nWidth, int nHeight)
{
	BOOL bResult = __super::SetActualBound(nX, nY, nWidth, nHeight);
	
	if(::IsWindow(m_hWndAdaptee) == TRUE)
	{
		CPoint ptParentAbsolutePos;
		GetParentAbsolutePosition(&ptParentAbsolutePos);

		::MoveWindow(m_hWndAdaptee, ptParentAbsolutePos.x + nX, ptParentAbsolutePos.y + nY, nWidth, nHeight, TRUE);
	}
	return bResult;
}
void CNWindowAdapter::OnParentVisibleChanged(CComponent* parent, BOOL bVisible)
{
	int nCmdShow = bVisible ? SW_SHOW : SW_HIDE;

	if(::IsWindow(m_hWndAdaptee) == TRUE)
	{
		::ShowWindow(m_hWndAdaptee, nCmdShow);
	}
}
void CNWindowAdapter::SetVisible(BOOL bVisible)
{
	__super::SetVisible(bVisible);

	int nCmdShow = bVisible ? SW_SHOW : SW_HIDE;

	if(::IsWindow(m_hWndAdaptee) == TRUE)
	{
		::ShowWindow(m_hWndAdaptee, nCmdShow);
	}	
}

HWND CNWindowAdapter::GetWindowAdaptee()
{
	return m_hWndAdaptee;
}

BOOL CNWindowAdapter::EnableWindow( BOOL bEnabled )
{
	BOOL bResult = __super::EnableWindow(bEnabled);

	if(::IsWindow(m_hWndAdaptee) == TRUE)
	{
		bResult = ::EnableWindow(m_hWndAdaptee, bEnabled);		
	}

	return bResult;
}
HWND CNWindowAdapter::CreateExWindow(	LPTSTR szClassName, DWORD dwStyle, HWND hWndParent /*= NULL*/, UINT uID /*= NULL*/, 
										DWORD dwStyleEx /*= NULL */ )
{
	if (uID == NULL)
	{
		uID = m_sCtrlId;
		m_sCtrlId++;
	}

	m_hWndAdaptee = ::CreateWindowEx(dwStyleEx, szClassName, NULL, dwStyle, 0, 0, 0, 0, hWndParent, (HMENU)uID, NULL, NULL);
	ATLASSERT(::IsWindow(m_hWndAdaptee) == TRUE);
	return m_hWndAdaptee;
}

UINT CNWindowAdapter::GetCtrlID()
{
	ATLASSERT(::IsWindow(m_hWndAdaptee) == TRUE);

	UINT uCtrlID = ::GetWindowLong(m_hWndAdaptee, GWL_ID);
	return uCtrlID;
}